@@ -16,21 +16,33 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import sys
+import os
+import getpass
 
 from monkeysign.ui import MonkeysignUi
+import monkeysign.translation
 
 class MonkeysignCli(MonkeysignUi):
-    """Sign a key in a safe fashion.
+    """sign a key in a safe fashion.
 
-This command should sign a key based on the fingerprint or user id
+This command signs a key based on the fingerprint or user id
 specified on the commandline, encrypt the result and mail it to the
 user. This leave the choice of publishing the certification to that
-person and makes sure that person owns the identity signed. This
-script assumes you have gpg-agent configure to prompt for passwords."""
+person and makes sure that person owns the identity signed.
+
+This program assumes you have gpg-agent configured to prompt for
+passwords."""
 
     # override default options to allow passing a keyid
-    usage = usage='%prog [options] <keyid>'
-    epilog='<keyid>: a GPG fingerprint or key id'
+    usage = _('%prog [options] <keyid>')
+    epilog = _('<keyid>: a GPG fingerprint or key id')
+
+    def parse_args(self, args):
+        """override main parsing: we absolutely need an argument"""
+        parser = MonkeysignUi.parse_args(self, args)
+        if self.pattern is None:
+            parser.print_usage()
+            sys.exit(_('wrong number of arguments, use -h for full help'))
 
     def main(self):
         """main code execution loop
@@ -38,18 +50,19 @@ script assumes you have gpg-agent configure to prompt for passwords."""
         we expect to have the commandline parsed for us
         """
 
-        if self.pattern is None:
-            sys.exit('wrong number of arguments')
-
         MonkeysignUi.main(self)
 
+        if not 'GPG_TTY' in os.environ:
+            os.environ['GPG_TTY'] = os.popen('tty').read()
+            self.log(_('reset GPG_TTY to %s') % os.environ['GPG_TTY'])
+
         # 1. fetch the key into a temporary keyring
         self.find_key()
 
         # 2. copy the signing key secrets into the keyring
         self.copy_secrets()
 
-        self.warn("Preparing to sign with this key\n\n%s" % self.signing_key)
+        self.warn(_('Preparing to sign with this key\n\n%s') % self.signing_key)
 
         # 3. for every user id (or all, if -a is specified)
         # 3.1. sign the uid, using gpg-agent
@@ -65,24 +78,33 @@ script assumes you have gpg-agent configure to prompt for passwords."""
         # implicit
 
     def yes_no(self, prompt, default = None):
-        ans = raw_input(prompt)
+        ans = raw_input(prompt.encode('utf-8'))
         while default is None and ans.lower() not in ["y", "n"]:
             ans = raw_input(prompt)
         if default: return default
         else: return ans.lower() == 'y'
 
+    def prompt_line(self, prompt):
+        return raw_input(prompt.encode('utf-8'))
+
+    def prompt_pass(self, prompt):
+        return getpass.getpass(prompt)
+
     def choose_uid(self, prompt, key):
         """present the user with a list of UIDs and let him choose one"""
-        allowed_uids = []
-        for uid in key.uidslist:
-            allowed_uids.append(uid.uid)
+        try:
+            allowed_uids = []
+            for uid in key.uidslist:
+                allowed_uids.append(uid.uid)
 
-        prompt += ' (1-%d or full UID, control-c to abort): ' % len(allowed_uids)
+            prompt += _(' (1-%d or full UID, control-c to abort): ') % len(allowed_uids)
 
-        pattern = raw_input(prompt)
-        while not (pattern in allowed_uids or (pattern.isdigit() and int(pattern)-1 in range(0,len(allowed_uids)))):
-            print "invalid uid"
             pattern = raw_input(prompt)
-        if pattern.isdigit():
-            pattern = allowed_uids[int(pattern)-1]
-        return pattern
+            while not (pattern in allowed_uids or (pattern.isdigit() and int(pattern)-1 in range(0,len(allowed_uids)))):
+                print _('invalid uid')
+                pattern = raw_input(prompt)
+            if pattern.isdigit():
+                pattern = allowed_uids[int(pattern)-1]
+            return pattern
+        except KeyboardInterrupt:
+            return False
