@@ -18,7 +18,7 @@
 
 """Tests that hit the network.
 
-Those tests are in a seperate file to allow the base set of tests to
+Those tests are in a separate file to allow the base set of tests to
 be ran without internet access.
 """
 
@@ -29,41 +29,38 @@ import signal
 sys.path.append(os.path.dirname(__file__) + '/..')
 
 from monkeysign.gpg import TempKeyring
+from test_lib import TestTimeLimit, AlarmException
 
-class AlarmException(IOError):
-    pass
-
-def handle_alarm(signum, frame):
-    raise AlarmException('timeout in %s' % frame.f_code.co_name)
-
-class TestGpgNetwork(unittest.TestCase):
-    """Seperate test cases for functions that hit the network
+class TestGpgNetwork(TestTimeLimit):
+    """Separate test cases for functions that hit the network
 
 each test needs to run under a specific timeout so we don't wait on
 the network forever"""
 
-    timeout = 3
-
     def setUp(self):
         self.gpg = TempKeyring()
         self.gpg.context.set_option('keyserver', 'pool.sks-keyservers.net')
-
-        signal.signal(signal.SIGALRM, handle_alarm)
-        signal.setitimer(signal.ITIMER_REAL, self.timeout)
+        TestTimeLimit.setUp(self)
 
     def test_fetch_keys(self):
         """test key fetching from keyservers"""
-        self.assertTrue(self.gpg.fetch_keys('4023702F'))
+        try:
+            self.assertTrue(self.gpg.fetch_keys('4023702F'))
+        except AlarmException:
+            raise unittest.case._ExpectedFailure(sys.exc_info())
 
     def test_special_key(self):
         """test a key that sign_key had trouble with"""
         self.assertTrue(self.gpg.import_data(open(os.path.dirname(__file__) + '/96F47C6A.asc').read()))
         self.assertTrue(self.gpg.import_data(open(os.path.dirname(__file__) + '/96F47C6A-secret.asc').read()))
-        self.assertTrue(self.gpg.fetch_keys('3CCDBB7355D1758F549354D20B123309D3366755'))
-        self.assertTrue(self.gpg.sign_key('3CCDBB7355D1758F549354D20B123309D3366755'))
+        try:
+            self.assertTrue(self.gpg.fetch_keys('3CCDBB7355D1758F549354D20B123309D3366755'))
+        except AlarmException:
+            raise unittest.case._ExpectedFailure(sys.exc_info())
+        self.assertTrue(self.gpg.sign_key('3CCDBB7355D1758F549354D20B123309D3366755', True))
 
     def tearDown(self):
-        signal.alarm(0)
+        TestTimeLimit.tearDown(self)
         del self.gpg
 
 if __name__ == '__main__':
